use std::collections::HashMap;

pub struct URL<'a> {
    pub raw: &'a str,
    pub path: &'a str,
    pub query: HashMap<&'a str, &'a str>,
}

impl<'a> URL<'a> {
    pub fn new(url: &'a str) -> Option<Self> {
        let raw = url;

        // Query string parsing
        let mut query = HashMap::new();
        let mut split = url.split('?');
        let path = split.nth(0)?;
        let query_string = split.next();

        for pair in query_string.unwrap_or("").split('&') {
            let mut key_value = pair.split('=');
            let key = match key_value.next() {
                Some(key) => key,
                None => continue,
            };
            let value = key_value.next().unwrap_or("");
            query.insert(key, value);
        }

        // Drop trailing slash but not for root path
        let path = if path.ends_with("/") && path.len() > 1 {
            &path[..path.len() - 1]
        } else {
            path
        };

        Some(URL { raw, path, query })
    }
}
