use std::error::Error;
use std::io::prelude::*;
use std::net::TcpStream;

type UnitOrBoxedError = Result<(), Box<dyn Error>>;

fn send_response(mut stream: TcpStream, status: u16, body: &str) -> UnitOrBoxedError {
    let status_line = match status {
        200 => "HTTP/1.1 200 OK",
        400 => "HTTP/1.1 400 BAD REQUEST",
        404 => "HTTP/1.1 404 NOT FOUND",
        405 => "HTTP/1.1 405 METHOD NOT ALLOWED",
        _ => "HTTP/1.1 500 INTERNAL SERVER ERROR",
    };

    let response = format!(
        "{}\r\nContent-Length: {}\r\n\r\n{}",
        status_line,
        body.len(),
        body
    );

    match stream.write(response.as_bytes()) {
        Ok(_) => {}
        Err(e) => {
            return Err(Box::new(e));
        }
    }

    match stream.flush() {
        Ok(_) => {}
        Err(e) => {
            return Err(Box::new(e));
        }
    }

    Ok(())
}

pub fn ok(stream: TcpStream, body: &str) -> UnitOrBoxedError {
    send_response(stream, 200, body)
}

pub fn send_400(stream: TcpStream) -> UnitOrBoxedError {
    send_response(stream, 400, "Bad Request")
}

pub fn send_404(stream: TcpStream) -> UnitOrBoxedError {
    send_response(stream, 404, "Not Found")
}

pub fn send_405(stream: TcpStream) -> UnitOrBoxedError {
    send_response(stream, 405, "Method Not Allowed")
}
